@@ -1,8 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#    Copyright (C) 2009 Daniel Fett
-#     This program is free software: you can redistribute it and/or modify
+#   Copyright (C) 2010 Daniel Fett
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
 #   (at your option) any later version.
@@ -15,7 +15,9 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-#    Author: Daniel Fett advancedcaching@fragcom.de
+#   Author: Daniel Fett agtl@danielfett.de
+#   Jabber: fett.daniel@jaber.ccc.de
+#   Bugtracker and GIT Repository: http://github.com/webhamster/advancedcaching
 #
 
 import geocaching
@@ -24,6 +26,102 @@ import geo
 import math
 import os
 
+usage = r'''Here's how to use this app:
+
+If you want to use the gui:
+%(name)s --simple
+    Simple User Interface, for mobile devices such as the Openmoko Freerunner
+%(name)s --desktop
+    Full User Interface, for desktop usage (not implemented yet)
+
+If you don't like your mouse:
+%(name)s set [options]
+        Change the configuration.
+%(name)s import [importactions]
+        Fetch geocaches from geocaching.com and write to the internal database.
+%(name)s import [importactions] do [actions]
+        Fetch geocaches from geocaching.com, put them into the internal database and do whatever actions are listed.
+%(name)s filter [filter-options] do [actions]
+        Query the internal database for geocaches and do the desired actions.
+%(name)s import [importactions] filter [filter-options] do [actions]
+        Import geocaches, put them into the internal database, filter the imported geocaches and run the actions.
+%(name)s sql "SELECT * FROM geocaches WHERE ... ORDER BY ... LIMIT ..." do [actions]
+        Select geocaches from local database and run the actions afterwards. Additional use of the filter is also supported. To get more information, run "%(name)s sql".
+options:
+        --user(name) username
+        --pass(word) password
+                Your geocaching.com login data.
+importactions:
+        --in coord1 coord2
+                Fetches the index of geocaches between the given coordinates.
+                These are interpreted as the corners of a rectangle. All caches
+                within the rectangle are retrieved. No details are retrieved.
+        --around coord radius-in-km
+                Fetches the index of geocaches at the given coordinate and radius
+                kilometers around it. No details are retrieved.
+        --at-route coord1 coord2 radius-in-km
+                Find caches along the route from coord1 to coord2. Uses OpenRouteService
+                and is not available for routes outside of europe.
+
+filter-options:
+        --in coord1 coord2
+        --around coord1 radius-in-km
+                See import actions.
+        -f|--found
+        -F|--not-found
+                Filter out geocaches which have (not) been found by the user.
+        -w|--was-downloaded
+                caches which have full detail information available
+
+        -s|--size (min|max) 1..4|micro|small|regular|huge|other
+                Specify a minimum or maximum size. If min/max is not given, show
+                only geocaches with the given size.
+        -d|--difficulty (min|max) 1.0..5.0
+        -t|--terrain (min|max) 1.0..5.0
+                Filter out geocaches by difficulty or terrain.
+        -T|--type type,type,...
+         type: virtual|regular|unknown|multi|event
+                Only show geocaches of the given type(s)
+        -o|--owner owner-search-string
+        -n|--name name-search-string
+        -i|--id id-search-string
+                Search owner, name (title) or id of the geocaches.
+        --new
+                Caches which were downloaded in current session. Useful to
+                get alerted when new caches arrive.
+actions:
+        --print
+                Default action, prints tab-separated list of geocaches
+        --fetch-details
+                Downloads Descriptions etc. for selected geocaches
+        --export-html folder
+                Dumps HTML pages to given folder
+        --command command
+                Runs command if more than one geocache has survived the filtering.
+                The placeholder %%s is replaced by a shell-escaped list of geocaches.
+
+        Not implemented yet:
+        --export-gpx folder
+                Dumps geocaches into separate GPX files
+        --export-single-gpx file
+                Dumps selected geocaches into a single GPX file
+        --draw-map zoom file
+                Draws one big JPEG file with the positions of the selected geocaches
+        --draw-maps zoom folder [tiles]
+                Draws a small JPEG image for every geocache.
+
+Preferred format for coordinates:
+    'N49 44.111 E6 29.123'
+    or
+    'N49.123456 E6.043212'
+
+Instead of a coordinate, you may also query geonames.com for a place name.
+Just start the string with 'q:':
+    q:London
+    'q:Brisbane, Australia'
+
+'''
+
 class ParseError(Exception):
     def __init__(self, errormsg, token = None):
         self.msg = errormsg
@@ -41,19 +139,19 @@ class RunError(Exception):
         return repr(self.msg)
 
 class Cli():
-    USES = ['geonames']
+    USES = ['geonames'] 
 
     # operators
     EQ = 0
     MIN = 1
     MAX = 2
 
-    def __init__(self, core, pointprovider, userpointprovider, dataroot):
+    def __init__(self, core, dataroot):
         self.nt = 1
         self.core = core
         self.caches = None
         self.new_caches = []
-        self.pointprovider = pointprovider
+        self.pointprovider = core.pointprovider
         pass
         
     def write_settings(self, settings):
@@ -65,13 +163,13 @@ class Cli():
         
         try:
             self.parse_input()
-        except ParseError as e:
+        except ParseError, e:
             if e.token == None:
                 print "# Parse Error at token '%s': " % sys.argv[self.nt - 1]
             else:
                 print "# Parse Error after Token '%s':" % sys.argv[e.token]
             print "# %s" % e.msg
-        except RunError as e:
+        except RunError, e:
             print "# Execution Error at token '%s': " % sys.argv[self.nt - 1]
             print "# %s" % e.msg
             
@@ -178,15 +276,15 @@ class Cli():
             elif token == '-w' or token == '--was-downloaded':
                 self.add_filter_has_details(True)
             elif token == '-s' or token == '--size':
-                op = self.parse_minmax()
+                op = self.parse_operator()
                 size = self.parse_size()
                 self.add_filter_size(op, size)
             elif token == '-d' or token == '--difficulty':
-                op = self.parse_minmax()
+                op = self.parse_operator()
                 diff = self.parse_decimal()
                 self.add_filter_difficulty(op, diff)
             elif token == '-t' or token == '--terrain':
-                op = self.parse_minmax()
+                op = self.parse_operator()
                 terr = self.parse_decimal()
                 self.add_filter_terrain(op, terr)
             elif token == '-T' or token == '--type':
@@ -222,11 +320,12 @@ class Cli():
                 self.action_fetch_details()
             elif token == '--export-html':
                 folder = self.parse_string()
-                self.action_export_html(folder)
+                self.action_export('html', folder)
             elif token == '--export-gpx':
                 folder = self.parse_string()
-                self.action_export_gpx(folder)
+                self.action_export('gpx', folder)
             elif token == '--export-single-gpx':
+                raise ParseError("Exporting to a single gpx file is currently not supported, sorry.")
                 filename = self.parse_string()
                 self.action_export_single_gpx(filename)
             elif token == '--draw-map':
@@ -259,12 +358,12 @@ class Cli():
             query = text[2:]
             try:
                 c = self.core.get_coord_by_name(query)
-            except Exception as e:
+            except Exception, e:
                 raise ParseError(e)
         else:
             try:
                 c = geo.try_parse_coordinate(text)
-            except Exception as e:
+            except Exception, e:
                 raise ParseError(e)
             
         return c
@@ -456,15 +555,12 @@ class Cli():
             self.core.on_download_cache(c)
             i += 1
     
-    def action_export_html(self, folder):
-        i = 1 
+    def action_export(self, format, folder):
+        i = 1
         for c in self.caches:
-            print "* (%d of %d)\tExporting '%s'" % (i, len(self.caches), c.title)
-            self.core.on_export_cache(c, folder)
+            print "* (%d of %d)\tExporting to %s: '%s'" % (i, len(self.caches), format, c.title)
+            self.core.on_export_cache(c, format, folder)
             i += 1
-    
-    def action_export_gpx(self):
-        pass
 
     def action_command(self, commandline):
         import unicodedata
